import urllib, sys, os, shutil, re, time, xbmcaddon, xbmcplugin, xbmcgui, xbmc
MODE_SEARCH_SONGS = 1
MODE_SEARCH_ALBUMS = 2
MODE_SEARCH_ARTISTS = 3
MODE_POPULAR_SONGS = 4
MODE_FAVORITES = 5
MODE_PLAYLISTS = 6
MODE_ALBUM = 7
MODE_ARTIST = 8
MODE_PLAYLIST = 9
MODE_SONG = 11
MODE_FAVORITE = 12

ACTION_MOVE_LEFT = 1
ACTION_MOVE_UP = 3
ACTION_MOVE_DOWN = 4
ACTION_PAGE_UP = 5
ACTION_PAGE_DOWN = 6
ACTION_SELECT_ITEM = 7
ACTION_PREVIOUS_MENU = 10

baseDir = os.getcwd()
resDir = xbmc.translatePath(os.path.join(baseDir, 'resources'))
libDir = xbmc.translatePath(os.path.join(resDir,  'lib'))
imgDir = xbmc.translatePath(os.path.join(resDir,  'img'))
cacheDir = os.path.join(xbmc.translatePath('special://masterprofile/addon_data/'), os.path.basename(os.getcwd()))
thumbDirName = 'thumb'
thumbDir = os.path.join('special://masterprofile/addon_data/', os.path.basename(os.getcwd()), thumbDirName)

baseModeUrl = 'plugin://plugin.audio.groove/'
playlistUrl = baseModeUrl + '?mode=' + str(MODE_PLAYLIST)
playlistsUrl = baseModeUrl + '?mode=' + str(MODE_PLAYLISTS)
favoritesUrl = baseModeUrl + '?mode=' + str(MODE_FAVORITES)

thumbDef = os.path.join(imgDir, 'default.tbn')

sys.path.append (libDir)
from GroovesharkAPI import GrooveAPI

try:
    groovesharkApi = GrooveAPI()
    if groovesharkApi.pingService() != True:
        raise StandardError('No Grooveshark service')
except:
    dialog = xbmcgui.Dialog()
    dialog.ok('Grooveshark XBMC', 'Unable to connect with Grooveshark.', 'Please try again later')
    sys.exit(-1)
  

class _Info:
    def __init__( self, *args, **kwargs ):
        self.__dict__.update( kwargs )

     
class Groveshark:
    
    albumImg = xbmc.translatePath(os.path.join(imgDir, 'album.png'))
    artistImg = xbmc.translatePath(os.path.join(imgDir, 'artist.png'))
    favoritesImg = xbmc.translatePath(os.path.join(imgDir, 'favorites.png'))
    playlistImg = xbmc.translatePath(os.path.join(imgDir, 'playlist.png'))
    popularSongsImg = xbmc.translatePath(os.path.join(imgDir, 'popularSongs.png'))
    songImg = xbmc.translatePath(os.path.join(imgDir, 'song.png'))
    defImg = xbmc.translatePath(os.path.join(imgDir, 'default.tbn'))
    fanImg = xbmc.translatePath(os.path.join(baseDir, 'fanart.png'))

    settings = xbmcaddon.Addon(id='plugin.audio.groove')
    songsearchlimit = int(settings.getSetting('songsearchlimit'))
    albumsearchlimit = int(settings.getSetting('albumsearchlimit'))
    artistsearchlimit = int(settings.getSetting('artistsearchlimit'))
    username = settings.getSetting('username')
    password = settings.getSetting('password')
    userid = 0
    
    def __init__( self ):
        self._handle = int(sys.argv[1])
        if os.path.isdir(cacheDir) == False:
            os.makedirs(cacheDir)
            xbmc.log("Made " + cacheDir)
        if os.path.isdir(thumbDir) == False:
            arttDir = os.path.join(cacheDir, thumbDirName)
            os.makedirs(arttDir)
            xbmc.log("Made " + arttDir)

    # Top-level menu
    def categories(self):

        self.userid = self._get_login()
        
        # Setup
        xbmcplugin.setPluginFanart(int(sys.argv[1]), self.fanImg)
        
        self._add_dir('Search songs...', '', MODE_SEARCH_SONGS, self.songImg, 0)
        self._add_dir('Search albums...', '', MODE_SEARCH_ALBUMS, self.albumImg, 0)
        self._add_dir('Search artists...', '', MODE_SEARCH_ARTISTS, self.artistImg, 0)
        self._add_dir('Popular songs', '', MODE_POPULAR_SONGS, self.popularSongsImg, 0)
        if (self.userid != 0):
            self._add_dir('Favorites', '', MODE_FAVORITES, self.favoritesImg, 0)
            self._add_dir('Playlists', '', MODE_PLAYLISTS, self.playlistImg, 0)

    # Search for songs            
    def searchSongs(self):
        query = self._get_keyboard(default="", heading="Search songs")
        if (query != ''):
            songs = groovesharkApi.getSongSearchResults(query, limit = self.songsearchlimit)
            if (len(songs) > 0):
                self._add_songs_directory(songs)
            else:
                dialog = xbmcgui.Dialog()
                dialog.ok('Grooveshark XBMC', 'No matching songs.')
                self.categories()
        else:
            self.categories()
    
    # Search for albums
    def searchAlbums(self):
        query = self._get_keyboard(default="", heading="Search albums")
        if (query != ''): 
            albums = groovesharkApi.getAlbumSearchResults(query, limit = self.albumsearchlimit)
            if (len(albums) > 0):
                self._add_albums_directory(albums)
            else:
                dialog = xbmcgui.Dialog()
                dialog.ok('Grooveshark XBMC', 'No matching albums.')
                self.categories()
        else:
            self.categories()
    
    # Search for artists
    def searchArtists(self):
        query = self._get_keyboard(default="", heading="Search artists")
        if (query != ''): 
            artists = groovesharkApi.getArtistSearchResults(query, limit = self.artistsearchlimit)
            if (len(artists) > 0):
                self._add_artists_directory(artists)
            else:
                dialog = xbmcgui.Dialog()
                dialog.ok('Grooveshark XBMC', 'No matching artists.')
                self.categories()
        else:
            self.categories()
                    
    # Get my favorites
    def favorites(self):
        userid = self._get_login()
        if (userid != 0):
            favorites = groovesharkApi.getUserFavoriteSongs()
            if (len(favorites) > 0):
                self._add_songs_directory(favorites)
            else:
                dialog = xbmcgui.Dialog()
                dialog.ok('Grooveshark XBMC', 'You have no favorites.')
                self.categories()
    
    # Get popular songs
    def popularSongs(self):
        popular = groovesharkApi.getPopularSongsToday(limit = self.songsearchlimit)
        if (len(popular) > 0):
            self._add_songs_directory(popular)
        else:
            dialog = xbmcgui.Dialog()
            dialog.ok('Grooveshark XBMC', 'No popular songs.')
            self.categories()

    # Get my playlists             
    def playlists(self):
        userid = self._get_login()
        if (userid != 0):
            playlists = groovesharkApi.getUserPlaylists()
            if (len(playlists) > 0):
                self._add_playlists_directory(playlists)
            else:
                dialog = xbmcgui.Dialog()
                dialog.ok('Grooveshark XBMC', 'You have no Grooveshark playlists.')
                self.categories()
        else:
            dialog = xbmcgui.Dialog()
            dialog.ok('Grooveshark XBMC', 'You must be logged in ', ' to see your Grooveshark playlists.')
                
    # Make songs a favorite 
    def favorite(self, songid):
        userid = self._get_login()
        if (userid != 0):
            xbmc.log("Favorite song: " + str(songid))
            groovesharkApi.addUserFavoriteSong(songID = songid)
            xbmc.executebuiltin('XBMC.Notification(Grooveshark XBMC, Added to favorites, 1000, ' + thumbDef + ')')
        else:
            dialog = xbmcgui.Dialog()
            dialog.ok('Grooveshark XBMC', 'You must be logged in', 'to add favorites.')

    # Show selected album
    def album(self, albumid):
        album = groovesharkApi.getAlbumSongs(albumid, limit = self.songsearchlimit)
        self._add_songs_directory(album)

    # Show selected artist
    def artist(self, artistid):
        albums = groovesharkApi.getArtistAlbums(artistid, limit = self.albumsearchlimit)
        self._add_albums_directory(albums)
    
    # Show selected playlist
    def playlist(self, playlistid):
        userid = self._get_login()
        if (userid != 0):
            songs = groovesharkApi.getPlaylistSongs(playlistid)
            self._add_songs_directory(songs)
        else:
            dialog = xbmcgui.Dialog()
            dialog.ok('Grooveshark XBMC', 'You must be logged in', 'to get Grooveshark playlists.')
            
    # Play a song
    def playSong(self, item):
        songid = item.getProperty('songid')
        song = groovesharkApi.getSongURLFromSongID(songid)
        if os.path.isfile(song):
            item.setPath(song)
            xbmc.log("Playing: " + song)
            xbmcplugin.setResolvedUrl(handle=int(sys.argv[1]), succeeded=True, listitem=item)
        else:
            xbmc.executebuiltin('XBMC.Notification(Grooveshark XBMC, Cannot play song, 1000, ' + thumbDef + ')')
    
    # Make a song directory item
    def songItem(self, songid, name, album, artist, coverart):
        songImg = self._get_icon(coverart, 'song-' + str(songid) + "-image")
        item = xbmcgui.ListItem(label = artist + " - " + album + " - " + name, thumbnailImage=songImg, iconImage=songImg)
        item.setInfo( type="music", infoLabels={ "title": name, "album": album, "artist": artist} )
        item.setProperty('mimetype', 'audio/mpeg')
        item.setProperty("IsPlayable", "true")
        item.setProperty('songid', str(songid))
        item.setProperty('coverart', songImg)
        item.setProperty('title', name)
        item.setProperty('album', album)
        item.setProperty('artist', artist)
        
        return item
    
    # Get keyboard input
    def _get_keyboard(self, default="", heading="", hidden=False):
        kb = xbmc.Keyboard(default, heading, hidden)
        kb.doModal()
        if (kb.isConfirmed()):
            return unicode(kb.getText(), "utf-8")
        return ''
    
    # Login to grooveshark
    def _get_login(self):
        if (self.username == "" or self.password == ""):
            dialog = xbmcgui.Dialog()
            dialog.ok('Grooveshark XBMC', 'Unable to login.', 'Check username and password in settings.')
            return 0
        else:
            if self.userid == 0:
                uid = groovesharkApi.login(self.username, self.password)
            if (uid != 0):
                return uid
            else:
                dialog = xbmcgui.Dialog()
                dialog.ok('Grooveshark XBMC', 'Unable to login.', 'Check username and password in settings.')
                return 0
    
    # Get a song directory item
    def _get_song_item(self, song):
        name = song[0]
        songid = song[1]
        album = song[2]
        artist = song[4]
        coverart = song[6]
        return self.songItem(songid, name, album, artist, coverart)            
        
    # File download            
    def _get_icon(self, url, songid):
        if url != 'None':
            localThumb = os.path.join(xbmc.translatePath(os.path.join(thumbDir, str(songid)))) + '.tbn'
            try:
                if os.path.isfile(localThumb) == False:
                    loc = urllib.URLopener()
                    loc.retrieve(url, localThumb)
            except:
                shutil.copy2(thumbDef, localThumb)
            return os.path.join(os.path.join(thumbDir, str(songid))) + '.tbn'
        else:
            return thumbDef
    
    # Add songs to directory
    def _add_songs_directory(self, songs):
        n = len(songs)
        xbmc.log("Found " + str(n) + " songs...")
        i = 0
        while i < n:
            song = songs[i]
            item = self._get_song_item(song)
            coverart = item.getProperty('coverart')
            songname = song[0]
            songid = song[1]
            songalbum = song[2]
            songartist = song[4]
            u=sys.argv[0]+"?mode="+str(MODE_SONG)+"&name="+urllib.quote_plus(songname)+"&id="+str(songid) \
            +"&album="+urllib.quote_plus(songalbum) \
            +"&artist="+urllib.quote_plus(songartist) \
            +"&coverart="+urllib.quote_plus(coverart)
            fav=sys.argv[0]+"?mode="+str(MODE_FAVORITE)+"&name="+urllib.quote_plus(songname)+"&id="+str(songid)
            menuItems = []
            menuItems.append(("Grooveshark favorite", "XBMC.RunPlugin("+fav+")"))
            item.addContextMenuItems(menuItems, replaceItems=False)
            xbmcplugin.addDirectoryItem(handle=int(sys.argv[1]),url=u,listitem=item,isFolder=False, totalItems=n)
            i = i + 1

        xbmcplugin.setContent(self._handle, 'songs')
        xbmcplugin.setPluginFanart(int(sys.argv[1]), self.fanImg)
    
    # Add albums to directory
    def _add_albums_directory(self, albums):
        n = len(albums)
        xbmc.log("Found " + str(n) + " albums...")
        i = 0
        while i < n:
            album = albums[i]
            albumArtistName = album[0]
            albumName = album[2]
            albumID = album[3]
            albumImage = self._get_icon(album[4], 'album-' + str(albumID))
            self._add_dir(albumName + " - " + albumArtistName, '', MODE_ALBUM, albumImage, albumID, n)
            i = i + 1
        xbmcplugin.setContent(self._handle, 'albums')
        xbmcplugin.addSortMethod(self._handle, xbmcplugin.SORT_METHOD_ALBUM_IGNORE_THE)
        xbmcplugin.setPluginFanart(int(sys.argv[1]), self.fanImg)
    
    # Add artists to directory
    def _add_artists_directory(self, artists):
        n = len(artists)
        xbmc.log("Found " + str(n) + " artists...")
        i = 0
        while i < n:
            artist = artists[i]
            artistName = artist[0]
            artistID = artist[1]
            self._add_dir(artistName, '', MODE_ARTIST, self.artistImg, artistID, n)
            i = i + 1
        xbmcplugin.setContent(self._handle, 'artists')
        xbmcplugin.addSortMethod(self._handle, xbmcplugin.SORT_METHOD_ARTIST_IGNORE_THE)
        xbmcplugin.setPluginFanart(int(sys.argv[1]), self.fanImg)

    # Add playlists to directory          
    def _add_playlists_directory(self, playlists):
        n = len(playlists)
        xbmc.log("Found " + str(n) + " playlists...")
        i = 0
        while i < n:
            playlist = playlists[i]
            playlistName = playlist[0]
            playlistID = playlist[1]
            self._add_dir(playlistName, '', MODE_PLAYLIST, self.playlistImg, playlistID, n)
            i = i + 1  
        xbmcplugin.setContent(self._handle, 'files')
        xbmcplugin.addSortMethod(self._handle, xbmcplugin.SORT_METHOD_PLAYLIST_ORDER)
        xbmcplugin.setPluginFanart(int(sys.argv[1]), self.fanImg)
      
    # Add whatever directory
    def _add_dir(self, name, url, mode, iconimage, id, items=1):
        
        u=sys.argv[0]+"?mode="+str(mode)+"&name="+urllib.quote_plus(name)+"&id="+str(id)
        dir=xbmcgui.ListItem(name, iconImage=iconimage, thumbnailImage=iconimage)
        dir.setInfo( type="Music", infoLabels={ "title": name } )
        return xbmcplugin.addDirectoryItem(handle=int(sys.argv[1]),url=u,listitem=dir,isFolder=True, totalItems=items)


# Parse URL parameters
def get_params():
    param=[]
    paramstring=sys.argv[2]
    xbmc.log(paramstring)
    if len(paramstring)>=2:
        params=sys.argv[2]
        cleanedparams=params.replace('?','')
        if (params[len(params)-1]=='/'):
            params=params[0:len(params)-2]
        pairsofparams=cleanedparams.split('&')
        param={}
        for i in range(len(pairsofparams)):
            splitparams={}
            splitparams=pairsofparams[i].split('=')
            if (len(splitparams))==2:
                param[splitparams[0]]=splitparams[1]
    return param
        
# Main
grooveshark = Groveshark();

params=get_params()
mode=None
try: mode=int(params["mode"])
except: pass
id=0
try: id=int(params["id"])
except: pass

# Call function for URL
if mode==None:
    grooveshark.categories()
       
elif mode==MODE_SEARCH_SONGS:
    grooveshark.searchSongs()
    
elif mode==MODE_SEARCH_ALBUMS:
    grooveshark.searchAlbums()

elif mode==MODE_SEARCH_ARTISTS:
    grooveshark.searchArtists()

elif mode==MODE_POPULAR_SONGS:
    grooveshark.popularSongs()

elif mode==MODE_FAVORITES:
    grooveshark.favorites()

elif mode==MODE_PLAYLISTS:
    grooveshark.playlists()

elif mode==MODE_SONG:
    try: name=urllib.unquote_plus(params["name"])
    except: pass
    try: album=urllib.unquote_plus(params["album"])
    except: pass
    try: artist=urllib.unquote_plus(params["artist"])
    except: pass
    try: coverart=urllib.unquote_plus(params["coverart"])
    except: pass
    song = grooveshark.songItem(id, name, album, artist, coverart)
    grooveshark.playSong(song)

elif mode==MODE_ARTIST:
    grooveshark.artist(id)
    
elif mode==MODE_ALBUM:
    grooveshark.album(id)
    
elif mode==MODE_PLAYLIST:
    grooveshark.playlist(id)
    
elif mode==MODE_FAVORITE:
    grooveshark.favorite(id)

if mode < MODE_SONG:
    xbmcplugin.endOfDirectory(int(sys.argv[1]))
